import { Chalk } from 'chalk';

import { base } from "../base";
import * as ufx from "../ufx";
import * as fs from "../fs";

import Config from "./config";
import Home from "./home";
import KitLog from "./kitlog";
import Npm from "./npm";

import { Module } from "./module";
import { Utils } from "./utils";

export default interface Runtime extends KitLog {
  /**
   * 框架的命名空间 {String}
   */
  nameSpace: string,

  /**
   * 加载缓存目录中的模块
   * @desc 
   *  - 如果本地缓存目录中存在此模块并没有过期，则直接调用；
   *  - 如果此模块的本地缓存过期了，则先去npm仓库下载最新版本，然后再调用；
   *  - 如果此模块不存在，则先从npm仓库安装到本地，然后再调用。
   * 
   * @param moduleName 模块名
   * @return {Function} mod
   */
  require(moduleName: string): Promise<any>;

  /**
   * 获取模块最新的package信息
   * @desc 模块的更新逻辑与Runtime.reauire相同
   *  - 如果本地缓存目录中存在此模块并没有过期，则直接读取package.json；
   *  - 如果此模块的本地缓存过期了，则先去npm仓库下载最新版本，然后再读取package.json；
   *  - 如果此模块不存在，则先从npm仓库安装到本地，然后再读取package.json。
   * 
   * @param moduleName 模块名
   * @return {Object} package.json的JSON对象
   */
  requirePkg(moduleName: string): Promise<any>;

  /**
   * 安装一个模块或安装当前项目的依赖
   * 
   * @param {Object} {name, ...otherOpts} = options
   * - {String} [name?] 将被安装的模块名，不传参如此参数，则表示安装当前项目的依赖
   * - {Object} [otherOpts] npminstall模块的Options
   * @see https://www.npmjs.com/package/npminstall
   * 
   */
  npmInstall(options?: any): Promise<void>;
  /**
   * 读取ufe模块
   * @desc 仅仅ufe cli的命名空间下存在此方法，是为了兼容历史版本ufx内核
   * 
   * @param name 模块名
   * @since ufe（仅阿里内网）
   * 
   * @return {Object}
   *  - mod {Function} - 模块方法
   *  - pkg {Object} - 模块的package.json对象
   */
  getUfeModule?(name: string): Promise<any>;

  /**
   * 获取本地模块的node_modules目录绝对路径
   * @desc 仅仅ufe cli的命名空间下存在此方法，是为了兼容历史版本ufx内核
   * @since ufe（仅阿里内网）
   * 
   * @return {String}
   */
  getUfeModulesPath?(): string;
  /**
   * 获取当前项目的配置
   * @desc 读取 .{nameSpace}.conf.js 文件，通过此方法可以获取其中的配置内容
   * 
   * @param key 对应配置的key属性
   * 
   * @example
   * 例如 .ufx.conf.js 内容如下:
   * ```js
   * module.exports = {
   *   kit: 'ufx-kit-demo',
  *    kitConfig: {
  *        a: 1,
  *        b: 2,
  *     },
  *  };
  * 
  * console.log(Runtime.getModuleConfig()); // { a: 1, b: 2}
  * console.log(Runtime.getModuleConfig('kitConfig')); // { a: 1, b: 2}
  * console.log(Runtime.getModuleConfig('kit')); // 'ufx-kit-demo'
  * ```
  */
  getModuleConfig(key?: string): any;

  /**
   * 获取本地模块的node_modules目录绝对路径
   * 
   * @return {String}
   * 
   */
  getModulesPath(): string;

  /**
   * 复制目录及目录下的文件
   * @desc 此方法是文件目录全拷贝，递归算法
   * 
   * @param {object: core.DirCopy} options
   * - {String} options.src 源文件的绝对路径
   * - {String} options.dist 目标的绝对路径
   * - {Boolean} options.silent 静默操作，不输出log
   * - {Array} options.ignore 数组, 类似 gitignore 的写法
   * - {Function} options.filenameTransformer 函数, 文件名转换函数列表
   * - {String} options.encoding 默认 utf-8
   * 
   * @example
   * ```js
   * this.dirCopy({
   *   src: tplRoot,
   *   dist: cwd,
   *   ignore: [ 'node_modules', '.DS_Store', '.idea'],
   *   filenameTransformer(n: string) {
   *     let name = n;
   *     if (name.startsWith('__')) {
   *       name = name.replace('__', '');
   *     } else if (name.startsWith('_')) {
   *       name = name.replace('_', '.');
   *     } else if (name.startsWith('.ufx')) {
   *       name = name.replace('ufx', this.nameSpace);
   *     }
   *     return name;
   *   },
   * });
   * ```
   */
  dirCopy(options: fs.DirCopy): void;

  /**
   * 复制文件
   * 
   * @param {object} options:<DirCopy>
   * - {String} options.src 目标文件的绝对路径
   * - {String} options.dist 绝对路径，可以是文件绝对路径，也可以是该文件所在的目录
   * - {Boolean} options.silent 静默操作，不输出log
   * - {Array} options.ignore 数组, 类似 gitignore 的写法
   * - {Function} options.filenameTransformer 函数, 文件名转换函数列表
   * - {String} options.encoding 默认 utf-8
   * 
   */
  fileCopy(options: fs.DirCopy): void;

  /**
   * .{nameSpace}.conf.js 文件的控制对象
   * @member {Object} core#modules/core-Config
   * 
   * @property {Function} exist(root?: string): boolean;
   * @property {Function} get(key: string, root?: string): any;
   * @property {Function} getAll(root?: string): any;
   * @property {Function} getKitName( root?: string): string | null;
   * @property {Function} setKitName(name: string): void;
   */
  config: Config;

  /**
   * Terminal string styling done right
   * 
   * @since Chalk
   * @see https://github.com/chalk/chalk
   */
  chalk: Chalk,

  /**
   * 框架的执行环境
   * 
   * @property {Function} isIntranet(): boolean;
   * @property {String} npmRegistry: string;
   * @property {String} isAlibabanet?: boolean;
   * @property {String} scope?: string;
   */
  env: ufx.RuntimeEnv;

  /**
   * node-fs-filesystem 拓展对象
   * @member {Object} core#modules/core-fs
   * 
   * @property {Function} dirCopy(options: DirCopy): void;
   * @property {Function} fileCopy(options: DirCopy): void;
   * @property {Function} move(oldPath: string, newPath: string): void;
   * @property {Function} remove(file: string): void;
   * 
   * @extends fs-extra
   * @see https://github.com/jprichardson/node-fs-extra
   */
  fs: fs.ufxFs | any;

  /**
   * 本地模块缓存目录的操控对象
   * @member {Object} core#modules/core-home
   * 
   * @property {Function} init(): void;
   * @property {Function} initHome(): void;
   * @property {Function} initHomePkg(): void;
   * @property {Function} initCore(): void;
   * @property {Function} cleanCoreDir(): void;
   * @property {Function} cleanCacheFile(): void;
   * @property {Function} getHomePath(): string;
   * @property {Function} getCorePath(): string;
   * @property {Function} getCoreModulePath(): string;
   * @property {Function} getCacheFilePath(): string;
   * @property {Function} getModulesPath(): string;
   * @property {Function} getModulePkgPath(fullName: string): string;
   * @property {Function} getModulePkgInfo(fullName: string): any;
   * @property {Function} cleanModulesDir(): void;
   */
  home: Home;

  /**
   * log函数方法
   * @desc 日志输出方法，一般建议直接调用具体的 log 函数，见下方
   * 
   * @return {Object}
   * - {Function} info(...content: any[]): void;
   * - {Function} success(...content: any[]): void;
   * - {Function} warn(...content: any[]): void;
   * - {Function} error(...content: any[]): void;
   * - {Function} debug: debugOpts & any;
   */
  log: ufx.logFn;

  /**
   * Npm安装、卸载、升级等操控对象
   * @member {Object} core#modules/core-npm
   * 
   * @property {Function} run(installer: string, paths: string | any[], options: object): Promise<any>;
   * @property {Function} get(name?: npmCommond): string;
   * @property {Function} install(pkg: string | any[], options?: object): Promise<any>;
   * @property {Function} getNpmInfo(name: string, options?: object): Promise<any>;
   * @property {Function} uninstall(pkg: string | any[], options?: object): Promise<any>;
   * @property {Function} update(pkg: string | any[], options?: object): Promise<any>;
   * @property {Function} installDependencies(options?: object): Promise<any>;
   * @property {Function} installModules(pkg: string | any[], options?: any): Promise<any>;
   * @property {Function} uninstallModules(pkg: string | any[], options?: any): Promise<any>;
   * @property {Function} check(name: string, options?: any): Promise<boolean>;
   */
  npm: Npm;

  /**
   * 框架模块的操控对象
   * @member {Object} core#modules/core-module
   * 
   * @property {Function} checkModule(kitName: string): Promise<checkResult>;
   * @property {Function} checkAndInstallModule(kitName: string): Promise<checkResult>;
   * @property {Function} require(moduleName: string): Promise<any>;
   * @property {Function} requirePkgInfo(moduleName: string): Promise<any>;
   * @property {Function} requireModule(moduleName: string): Promise<any>;
   * @property {Function} getModule(moduleName: string): Promise<any>;
   * @property {Function} modLocalExist(name: string): Promise<boolean>;
   * @property {Function} getLocalModuleList(): Promise<ListItem[]>;
   * @property {Function} getOnlineModuleList(keyword?: string): Promise<ListItem[]>;
   * @property {Function} getKitList(): Promise<ListItem[]>;
   * @property {Function} getAllModules(): Promise<ufxModules>;
   */
  module: Module;

  /**
   * 交互式命令行工具Inquirer
   * 
   * @extends Inquirer
   * @see https://github.com/SBoudrias/Inquirer.js#readme
   */
  inquirer?: any;

  /**
   * UFX框架的通用工具函数集
   * @member {Object} core#modules/core-utils
   * 
   * @property {Function} getUserHomeDir(): string | null;
   * @property {Function} getApiData(options: any): Promise<NetData>;
   * @property {Function} checkNetStatus(testUrl: string, timeout?: number): Promise<boolean>;
   * @property {Function} getFileNames(dir: fs.PathLike): string[];
   * @property {Function} formatArgv(argv: any): FormatArgv;
   * @property {Function} generateNames(str: string): string;
   * @property {Function} isEmptyObject(e: Object): boolean;
   * @property {Function} isDirectory(dir: fs.PathLike): boolean;
   * @property {Function} isFile(file: fs.PathLike): boolean;
   * @property {Function} isGeneratorFunction(obj: any): boolean;
   * @property {Function} getIp(): string;
   * @property {Function} getPkgName(name: string): string;
   * @property {Function} getFullName(name: string): string;
   * @property {Function} getKitName(name: string): string;
   * @property {Function} getPluginName(name: string): string;
   * @property {Function} getNpmSearchApi(name: string): string;
   * @property {Function} getFrameworkKeyword(): string;
   * @property {Function} checkModuleType(name: string): ModuleType;
   * @property {Function} printInfo(list: ListItem[]): void;
   * @property {Function} fixedWidth(str: string, width?: number, placeholder?: string): string
   */
  utils: Utils

  /**
   * CLI子命令数组
   * 
   * @example
   * ```bash
   * ufx dev web # Runtime.clientArgs = ['web']
   * ufx dev weex # Runtime.clientArgs = ['weex']
   * ```
   */
  clientArgs?: string[];

  /**
   * CLI命令的参数对象
   * 
   * @example
   * ```bash
   * ufx dev --port 8080 # Runtime.clientArgs = { port: 8080}
   * ufx dev --watch --ip 127.0.0.1 # Runtime.clientArgs = { watch: true, ip: '127.0.0.1' }
   * ```
   */
  clientOptions?: ufx.ufxObj<any>;

  // 同 Runtime.clientOptions
  clientOpts?: ufx.ufxObj<any>;
}

