import * as fs from "fs-extra"
import * as base from '../base/utils'

export interface FormatArgv {
  cliArgs: any[];
  cliOpts: object;
}

export type ModuleType = 'KIT' | 'PLUGIN' | 'NORMAL';


export interface ListItem {
  name: string;
  description: string;
}

export interface Utils extends base.Utils {

  /**
   * 格式化命令行参数
   * @param argv
   */
  formatArgv(argv: any): FormatArgv;

  /**
   * 格式化字符串为驼峰
   * @param {String} str
   * @return {String}
   * 
   * @example
   * ```js
   * expect(utils.generateNames('test-123')).toBe('test123');
   * expect(utils.generateNames('test')).toBe('test');
   * expect(utils.generateNames('test-sdf')).toBe('testSdf');
   * expect(utils.generateNames('test-zxc-asd')).toBe('testZxcAsd');
   * ```
   */
  generateNames(str: string): string;

  /**
   * 判断对象是否为空
   * @param {Object} e
   * @return {Boolean}
   */
  isEmptyObject(e: Object): boolean;

  /**
   * 判断路径是否为目录
   * @param dir
   * @return {Boolean}
   */
  isDirectory(dir: fs.PathLike): boolean;

  /**
   * 判断是否为文件
   * @param file
   * @return {Boolean}
   */
  isFile(file: fs.PathLike): boolean;

  /**
   * 判断当前对象是否为 generator 函数
   * @param {any} obj
   * @return {boolean}
   */
  isGeneratorFunction(obj: any): boolean;

  /**
   * 获取本机的ip地址
   * @return {string}
   */
  getIp(): string;

  /**
   * 根据传入的插件名称缩写,获取模块名称
   * @param name
   * @return {string}
   */
  getPkgName(name: string): string;
  
  /**
   * 根据传入的模块,获取模块的完整名称
   * @param {string} name
   * @return {string}
   */
  getFullName(name: string): string;
 
  /**
   * 获取套件的完整name
   * @param {string} name 套件名的缩写
   * @return {string}
   * 
   * @example
   * ```js
   * const name = 'asdfg';
   * expect(utils.getKitName(name)).toBe(`${nameSpace}-kit-${name}`);
   * expect(utils.getKitName(`kit-${name}`)).toBe(`${nameSpace}-kit-${name}`);
   * expect(utils.getKitName(`@ali/kit-${name}`)).toBe(`${nameSpace}-kit-${name}`);
   * ```
   */
  getKitName(name: string): string;

  /**
   * 获取插件的完整name
   * @param name 插件名的缩写
   * @return {string}
   * 
   * @example
   * ```js
   * const name = 'asdfg';
   * const pluginName = `${nameSpace}-plugin-${name}`;
   * expect(utils.getPluginName(name)).toBe(pluginName);
   * expect(utils.getPluginName(`plugin-${name}`)).toBe(pluginName);
   * expect(utils.getPluginName(`@ali/plugin-${name}`)).toBe(pluginName);
   * ```
   */
  getPluginName(name: string): string;

  /**
   * 生成搜索npm模块的api
   * @param name npm模块名
   * @return {string}
   */
  getNpmSearchApi(name: string): string;

  /**
   * 生成框架的keyword
   * @return {string}
   * 
   * @example
   * ```js
   * // ufx运行在阿里内网
   * expect(utils.getFrameworkKeyword()).toBe('@ali/ufx-');
   * // ufx.nameSpace == 'ufx'
   * expect(utils.getFrameworkKeyword()).toBe('ufx-');
   * ```
   */
  getFrameworkKeyword(): string;

  /**
   * 检查模块的类型，官方模块 or 普通模块
   * @param {string} name npm模块名
   * @return {string} NORMAL:普通模块 | PLUGIN: 插件 | KIT: 套件
   * 
   * @example
   * ```js
   * const name = 'asdfg';
   * expect(utils.checkModuleType(name)).toBe('NORMAL');
   * expect(utils.checkModuleType(`@xxx/${nameSpace}-plugin-${name}`)).toBe('NORMAL');
   * expect(utils.checkModuleType(`@xxx/${nameSpace}-kit-${name}`)).toBe('NORMAL');
   * expect(utils.checkModuleType(`${nameSpace}-plugin-${name}`)).toBe('PLUGIN');
   * expect(utils.checkModuleType(`${nameSpace}-kit-${name}`)).toBe('KIT');
   * ```
   */
  checkModuleType(name: string): ModuleType;

  /**
   * 打印framework的套件或模块列表
   * @param {Array} list
   */
  printInfo(list: ListItem[]): void;

  /**
   * 补充字符串对齐所需的中间空隙
   * @param {string} str
   * @param {number} width 字符补齐后的总长度
   * @return {string}
   * 
   * @example
   * ```js
   * const name = 'asdfg';
   * expect(utils.getFullName(name)).toBe(name);
   * const plugName = `${nameSpace}-plugin-${name}`;
   * expect(utils.getFullName(`plugin-${name}`)).toBe(plugName);
   * expect(utils.getFullName(`${nameSpace}-plugin-${name}`)).toBe(plugName);
   * expect(utils.getFullName(`@xxx/${nameSpace}-plugin-${name}`)).not.toBe(plugName);
   * const kitName = `${nameSpace}-kit-${name}`;
   * expect(utils.getFullName(`kit-${name}`)).toBe(kitName);
   * expect(utils.getFullName(`${nameSpace}-kit-${name}`)).toBe(kitName);
   * expect(utils.getFullName(`@xxx/${nameSpace}-kit-${name}`)).not.toBe(kitName);
   * ```
   */
  fixedWidth(str: string, width?: number, placeholder?: string): string;

  /**
   * 处理模块名,返回正确的名称
   * @param {string} moduleName 模块名, e.g: abc/abc.js、@xxx/abc/a.js、@xxx/abc、abc
   * @return {string}
   * 
   * @example
   * ```js
   * expect(utils.formatModuleName('abc/abc.js')).toBe('abc');
   * expect(utils.formatModuleName('abc')).toBe('abc');
   * expect(utils.formatModuleName('abc/def')).toBe('abc');
   * expect(utils.formatModuleName('@ali/abc/a.js')).toBe('@ali/abc');
   * expect(utils.formatModuleName('@ali/abc')).toBe('@ali/abc');
   * ```
   */ 
  formatModuleName(moduleName: string): string;
}
